#!/usr/bin/env python

from zope.interface import Interface, implements
from twisted.cred import portal, checkers, credentials, error as credential_error
from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
from twisted.web2 import server
from twisted.web2.auth import digest, wrapper
from twisted.web2.channel import http
from twisted.internet import reactor
from twisted.application import service, internet

from txjsonrpc.auth import wrapResource
from txjsonrpc.web2 import jsonrpc

import logging
import datetime
import time

import blitzortung

class PasswordDictChecker(object):
  implements(checkers.ICredentialsChecker)
  credentialInterfaces = (credentials.IUsernamePassword,)

  def __init__(self, passwords):
    self.passwords = passwords

  def requestAvatarId(self, credentials):
    username = credentials.username
    if username in self.passwords:
      if credentials.password == self.passwords[username]:
        return defer.succeed(username)
    return defer.fail(credential_error.UnathorizedLogin("invalid username/password"))

class IUserAvatar(Interface):
  ' should have attribute username '
 
class UserAvatar(object):
  implements(IUserAvatar)

  def __init__(self, username):
    self.username = username


class TestRealm(object):
  implements(portal.IRealm)

  def __init__(self, users):
    self.users = users

  def requestAvatar(self, avatarId, mind, *interfaces):
    if INamedUserAvatar in interfaces:
      logout = lambda: None
      return (IUserAvatar,
	  UserAvatar(avatarId),
	  logout)
    else:
      raise KeyError('none of the requested interfaces is supported')


class Blitzortung(jsonrpc.JSONRPC):
  """
  An example object to be published.
  """

  addSlash = True

  def jsonrpc_get_strokes(self, request, min_time, min_id=None):
    strokedb = blitzortung.db.Stroke()

    endtime = datetime.datetime.utcnow()
    starttime = endtime - datetime.timedelta(minutes=min_time)
    time_interval = blitzortung.db.TimeInterval(starttime, endtime)

    if min_id is not None:
      id_interval = blitzortung.db.IdInterval(min_id)
    else:
      id_interval = None
      min_id = 0

    area = None
    order = blitzortung.db.Order('id')

    reference_time = time.time()
    strokes = strokedb.select(time_interval, id_interval, area, order)

    query_time = time.time()

    max_id = None
    stroke_array = []
    for stroke in strokes:
      stroke_data = []

      timestamp = stroke.get_timestamp()
      stroke_data.append(timestamp.strftime("%Y%m%dT%H:%M:%S.%f")[:-3])
      stroke_data.append(timestamp.microsecond % 1000 * 1000 + stroke.get_timestamp_nanoseconds())
      stroke_data.append(stroke.get_x())
      stroke_data.append(stroke.get_y())
      stroke_data.append(stroke.get_lateral_error())
      stroke_data.append(stroke.get_amplitude())
      stroke_data.append(stroke.get_station_count())
      stroke_data.append(stroke.get_type())

      stroke_array.append(stroke_data)
      max_id = stroke.get_id()
  
    response = {'strokes': stroke_array}

    if max_id:
      response['next'] = long(max_id + 1)

    print '%s get_strokes(%d, %d): #%d (%.2fs)' %(request.remoteAddr.host, min_time, min_id, len(strokes), query_time - reference_time)

    return response

  def jsonrpc_get_stations(self, request):
    stationsdb = blitzortung.db.Station()

    reference_time = time.time()
    stations = stationsdb.select()
    query_time = time.time()

    station_array = []
    for station in stations:
      station_data = []

      station_data.append(station.get_number())
      station_data.append(station.get_location_name())
      station_data.append(station.get_country())
      station_data.append(station.get_x())
      station_data.append(station.get_y())
      if station.get_offline_since():
        station_data.append(station.get_offline_since().strftime("%Y%m%dT%H:%M:%S.%f")[:-3])

      station_array.append(station_data)
  
    response = {'stations': station_array}

    print '%s get_stations(): #%d (%.2fs)' %(request.remoteAddr.host, len(stations), query_time - reference_time)

    return response


users = {'test':'test'}

# Set up the application and the JSON-RPC resource.
application = service.Application("Blitzortung.org JSON-RPC Server")
root = Blitzortung()

# Define the credential checker the application will be using and wrap the JSON-RPC resource.
checker = InMemoryUsernamePasswordDatabaseDontUse()
checker.addUser('test','test')
realm_name = "Blitzortung.org JSON-RPC App"
wrappedRoot = wrapResource(root, [checker], realmName=realm_name)

# With the wrapped root, we can set up the server as usual.
site = server.Site(root)
chan = http.HTTPFactory(site)
jsonrpcServer = internet.TCPServer(7080, chan)
jsonrpcServer.setServiceParent(application)

